﻿$(function () {
	$("#toggleFilter").click(function () {
		$(".filterBox").slideToggle();

		//fill techs with ajax
		if ($("#handledByUserId").length && !$("#handledByUserId option").length) {
			FillTechsFilter();

			//also refresh the multiselect so it has proper width (b/c element was hidden, the width is not correct)
			$("#statusId").multiselect("refresh");
			$("#priorityId").multiselect("refresh");
		}

		if ($("#fromDepartmentId").length && !$("#fromDepartmentId option").length) {
			FillDepsFilter();
		}
	});

	//show more button
	InitMoreButton();

	//save ticket ids in array for prev/next
	SaveGridIdsInStorage();

	//saved filters (via VUE)
	$('#btnSaveFilter').click(function () { $("#saveFilterDiv").modal() });

	window.filterApp = Vue.createApp({
		data() { return { Filters: [] } },
		methods: {
			add: function (name) {
				if (!name) return;
				var curUrl = IndexPageModel.CurrentUrl; //get current url
				this.Filters.push({ FilterName: name, FilterUrl: curUrl.replace('filterPressed=True&', '') }); //add new filter-object to model
				$.modal.close(); //hide the dialog
				this.saveToSrv();
			},
			del: function (index) {
				this.Filters.splice(index, 1); //remove clicked index from model
				this.saveToSrv();
			},
			saveToSrv: function () {
				const formData = new FormData();
				formData.append("json", JSON.stringify(this.$data))
				fetch(GridApp.Url + "/Tickets/SaveUserFilters", {
					method: "post",
					body: formData
				})
			}
		},
		mounted: function () {
			fetch(GridApp.Url + "/Tickets/GetUserFilters")
				.then(response => response.text())
				.then(data => {
					if (data)
						this.Filters = JSON.parse(data).Filters;
					else
						this.Filters = localStorage.getItem("SavedFilters") ? JSON.parse(localStorage.getItem("SavedFilters")).Filters : [];
				});
		}
	}).mount("#divSavedFilters");

	// collapse tags if needed
	if ($("span", ".tagslist").length > 10) {
		$("span", ".tagslist").slice(10).hide(); // hide everything besides the first 10 tags
		var showMore = $("<span class='tag' title='Show more tags' style='cursor:pointer'><i class='fa fa-ellipsis-h'></i></span>"); // the button to show all tags
		$("span:nth-child(11)", ".tagslist").after(showMore);
		showMore.click(function (e) {
			$("span", ".tagslist").slice(10).fadeIn();
			showMore.hide();
		});
	}

	//clear emtpy field values from query string so we don't face "too log url" issue
	//also remove cookies b/c we're setting explicit filter values
	$("#filterForm").submit(function () {
		$("#filterForm").find("input[name],select[name]")
			.filter(function (index, input) {
				return !input.value;
			})
			.prop("name", "").prop("disabled", true); //"disabled" need for IE to prevent sending "&=&=&=&"
		$.removeCookie("dateFrom"); $.removeCookie("dateTo"); $.removeCookie("statusFilterId"); $.removeCookie("priorityId"); $.removeCookie("fromUserId"); $.removeCookie("handledByUserId"); $.removeCookie("fromCompanyId"); $.removeCookie("dueFilter");
	});

	var $divCategories = $('#divCategories'); //sticky categoris menu
	if ($divCategories.length && $divCategories[0].getBoundingClientRect().bottom > window.innerHeight) {//if cat menu ouside scroll - get rid of "sticky"
		$('#leftsidebar').css("position", "inherit");
	}

	if (IndexPageModel.IsTech)
		InitViewingTechs();

	//select all
	$('.cbAll').click(function () {
		if (this.checked && $("#hdnHasMoreTickets").val() == "true") {
			$('#divSelectAll').show();
			$('.cb').click(resetAllSelector);
		}
		else {
			resetAllSelector();
		}
		function resetAllSelector() {
			$('#divSelectAll').hide();
			$('#gridForm').attr('action', GridApp.Url + "/Tickets/BulkAction"); //restore form action to default
		}
	});

	//shift+click for category links
	$("a[data-categoryid]").click(function (event) {
		if (event.shiftKey) {
			var url = document.location.href;
			if (url.indexOf("sectionId=") > -1) //if we're currently looking at a section
				document.location.href = this.href; //then simply go to category
			else { //otherwise add category
				url += url.indexOf("?") > -1 ? "&" : "?";
				url += "categoryId=" + $(this).data("categoryid");
				document.location.href = url;
			}
			return false;
		}
		return true;
	});
	//shift+click for tag links
	$("a[data-tagid]").click(function (event) {
		if (event.shiftKey) {
			var url = document.location.href;
			url += url.indexOf("?") > -1 ? "&" : "?";
			url += "tagId=" + $(this).data("tagid");
			document.location.href = url;
			return false;
		}
		return true;
	});

	//also override default browser behavior on shift-clicking a SECTION
	$('li.section > a').click(function (event) {
		if (event.shiftKey) {
			document.location.href = this.href;
			return false;
		}
	});

	if (IndexPageModel.IsTech) {
		$(".ticketRow a.ticketLink").tooltipster({
			delay: 1500, maxWidth: 500, contentAsHTML: true, theme: 'my-custom-theme', position: 'top-left', content: "", updateAnimation: false,
			triggerClose: {
				mouseleave: true,
				scroll: true
			},
			functionBefore: function (origin, continueTooltip) {
				continueTooltip();

				if (origin.data('ajax') !== 'cached') {
					fetch(origin[0].href.replace("/Ticket/", "/Ticket/GetTicketBody/"))
						.then(function (response) { return response.text(); })
						.then(function(txt) {
							origin.tooltipster('content', txt).data('ajax', 'cached');
						})
						.catch(function () { }); //request failed, do nothing
				}
			}
		});
	}
});

function FillTechsFilter() {
	//fill techs with ajax
	$.ajax({
		url: globalRootAppUrl + 'Tickets/GetTechUsers',
		type: "POST",
		success: function (data) {
			var $selTechId = $('#handledByUserId');
			$selTechId.empty();
			$.each(data, function (i, item) {
				$selTechId.append("<option value='" + item.UserID + "'>" + item.FullNameAndLogin + "</option>");
			});
			var qs = getQueryString();
			if (qs["handledByUserId"]) $selTechId.val(qs["handledByUserId"]); //set selected tech if we are filtered by tech in querystring
			$("#handledByUserId").multiselect({ noneSelectedText: $("#handledByUserId").data("prompt"), header: false }); //setup multiselect
		}
	});
}

function FillDepsFilter() {
	//fill techs with ajax
	$.ajax({
		url: globalRootAppUrl + 'Tickets/GetDepartments',
		type: "POST",
		success: function (data) {
			var $selDepId = $('#fromDepartmentId');
			$selDepId.empty();
			$selDepId.append("<option value=''></option>");
			$.each(data, function (i, item) {
				$selDepId.append("<option value='" + item.DepartmentID + "'>" + item.Name + "</option>");
			});
			var qs = getQueryString();
			if (qs["fromDepartmentId"]) $selDepId.val(qs["fromDepartmentId"]); //set selected dep if we are filtered by dep in querystring
		}
	});
}

function InitMoreButton() {
	if ($("#hdnHasMoreTickets").val() == "true" && $(".trMore").length == 0) {
		var grid = $("#tblTickets");
		var colspan = $("tbody tr:nth-child(2) td", grid).length;
		$("tbody", grid).append("<tr class='trMore'><td onclick='ClickMore();' colspan='" + colspan + "'><a class='grey' href='javascript:;' onclick='return false;'><span class='progress'>▼&nbsp;&nbsp;&nbsp;</span>MORE TICKETS...</a></td></tr>");
	}
}

var loadingMoreInProgress = false;
function ClickMore() {
	if (loadingMoreInProgress) return false; // prevent double clicks

	loadingMoreInProgress = true;
	var more = $(".trMore");
	$(".progress", more).html("<img src='" + globalRootCdnUrl + "images/indicator.svg' />&nbsp;&nbsp;");

	var numTicketsShown = $(".ticketRow").length;
	GetTicketsFromServer(numTicketsShown, function (data, status, jqXHR) {
		more.remove(); //remove "more" btn
		var separator = "<tr class='separator'><td style='height: 10px;padding: 0' colspan='" + $("#tblTickets tbody tr:nth-child(2) td").length + "'></td></tr>";
		var html = separator + data;
		$("#tblTickets tr.ticketRow").last().after(html); // draw the tickets (append to end)

		$("#hdnHasMoreTickets").val(jqXHR.getResponseHeader("HasMoreTickets"));
		InitMoreButton(); //then reinit "more" button
		SaveGridIdsInStorage(); //save "more" tickets in storage
		loadingMoreInProgress = false;
	});

	//now add "p" parameter to URL so the browser remembers grid position when clicking "back"
	if (typeof (window.history.pushState) == 'function') { // check for modern browser
		var p = getQueryString()["p"] || 1;
		p = Number(p) + 1;
		var url = updateQueryStringParameter(document.location.href, "p", p);
		window.history.pushState(null, document.title, url);
	}
}

function ColumnVisibilyCheckboxChanged(elem) {
	elem = $(elem);
	var indexInGrid = $("#tblTickets th[data-serverindex='" + elem.val() + "']").index() + 1;
	if (elem.is(":checked")) {
		$("#tblTickets td:nth-child(" + indexInGrid + "), #tblTickets th:nth-child(" + indexInGrid + ")").show();
	} else {
		$("#tblTickets td:nth-child(" + indexInGrid + "), #tblTickets th:nth-child(" + indexInGrid + ")").hide();
	}
	SaveHiddenColumns();
}

function SaveHiddenColumns() {
	var hiddenStr = "";
	$("#divSelectColumns input:not(:checked)").each(function () {
		hiddenStr += $(this).val() + ",";
	});
	$.cookie("hiddenColumns", hiddenStr, { expires: 20, path: "/" });
}

function SaveGridIdsInStorage() {
	var ticketarr = $("#tblTickets tr[data-ticketid]").map(function () { return $(this).data("ticketid"); }).toArray(); // array of strings ticketIds
	sessionStorage.setItem("ticketIds", JSON.stringify(ticketarr));
}

function ShowSubtickets(plusBtn, ticketId) {
	$(plusBtn).toggleClass("fa-plus-square-o").toggleClass("fa-minus-square-o"); //toggle plus sign
	if ($('#ticketRow' + ticketId).nextUntil(":not(.subticket)").toggle().length) //already loaded? then toggle and exit
		return;

	var cfCount = $('#ticketRow' + ticketId + ' td.cf').length;

	$.post(globalRootAppUrl + 'Tickets/GetSubtickets?ticketId=' + ticketId + '&cfCount=' + cfCount + "&orderByCol=" + IndexPageModel.OrderByColumn, function (data) {
		if (!data) return;

		//removing dupe tickets
		//$(data).map(function () { var id = $(this).data("ticketid"); $('#ticketRow' + id).not('.subticket').remove(); });

		$('#ticketRow' + ticketId).after(data);
	});
}

/**
 * This function gets tickets from server and returns HTML of the tickets (via callback)
 * @param {number} numTicketsToSkip num tickets to skip
 * @param {Function} callback function that has the same signature as jQuery success callback: func(html, status, jqXHR)
 */
function GetTicketsFromServer(numTicketsToSkip, callback) {
	var url = IndexPageModel.GetMoreUrl;
	url += "&numTicketsShown=" + numTicketsToSkip;
	$.post(url, callback); //calling the callbak that DRAWS THE TICKETS
}

function RefreshCurrentGrid() {
	if (window.blockingFlag) return; //primitive locking implementation
	window.blockingFlag = true;

	//pause for 1 second so we "gather" several new ticket notifications
	setTimeout(function () {
		//call "GetTicketsFromServer" only ONCE per multiple browser tabs pinting to the same grid
		TabUtils.CallOnce("gridrefreshlock" + IndexPageModel.GetMoreUrl, function () { //we add "location.search" to the lock key so it unique to same grid
			GetTicketsFromServer(0, function (html, status, jqXHR) {
				TabUtils.BroadcastMessageToAllTabs("gridRefreshedEvent" + IndexPageModel.GetMoreUrl, { html: html, hasMore: jqXHR.getResponseHeader("HasMoreTickets") } ); //send the event to all tabs
			});
		});

		window.blockingFlag = false;
	}, 5000);
}

function InitViewingTechs() {
	if (!IndexPageModel.IsTech) return;
	$.get(GridApp.Url + "/Tickets/GetViewingTechsForAllTickets", function (data) {
		if (data && data.constructor === Array) {
			$.each(data, function (i, item) {
				updateViewingTechs(item.TicketID, item.ViewingTechs);
			});
		}
	});
}

function updateViewingTechs(ticketId, users) {
	var $ticketRow = $('#ticketRow' + ticketId);
	if (!$ticketRow.length) return; //ticket is not currently on screen

	if (users && users.length > 0) {
		$(".techsViewing", $ticketRow).fadeIn("fast").attr("title", users.join(", ") + (users.length > 1 ? " are " : " is ") + " looking at the ticket");
	} else {
		$(".techsViewing", $ticketRow).fadeOut("fast");
	}
}

function RenderTicket(ticketId) {
	window.renderticketlock = window.renderticketlock || [];
	if (window.renderticketlock.indexOf(ticketId) > -1) return; //multiple signalr events for same ticket
	window.renderticketlock.push(ticketId);
	$.post(GridApp.Url + "/Tickets/RenderTicket?ticketId", { ticketId: ticketId }, function (data) {
		$("#tblTickets tbody").prepend(data);
		updateTicketCount(1);

		// if the grid is not visible then an empty state is shown
		if (!$("#tblTickets").is(":visible")) {
			$("#noTicketsMessage").hide();
			$("#tblTickets").fadeIn();
		}
	}).always(function () {
		window.renderticketlock = window.renderticketlock.filter(function (item) { return item !== ticketId });
	});
}

//this handles the tab event
TabUtils.OnBroadcastMessage("gridRefreshedEvent" + IndexPageModel.GetMoreUrl, function (data) {
	if ($("#tblTickets").length == 0) { document.location.reload(); return; } //there is no tickets grid AT ALL (no tickets) - lets just refresh the page
	$("#tblTickets tr.ticketRow, #tblTickets .separator, #tblTickets .trMore").remove(); //remove existing tickets and more-button
	$("#tblTickets tbody").append(data.html); // draw the refreshed tickets
	SaveGridIdsInStorage(); //save ticket ids in array for prev/next

	$("#hdnHasMoreTickets").val(data.hasMore);
	InitMoreButton(); //then reinit "more" button

	var newNuTicketsShown = $("#tblTickets .ticketRow").length;
	if (newNuTicketsShown < IndexPageModel.Limit) {
		//update counters in subtabs (only if less than "Pagesize", because otherwise it will be 50 no matter the actual number)
		var $activetab = $('#divBigHeader li.active2 a');
		var tabtxt = $activetab.text();
		if (tabtxt.indexOf("(") > -1) tabtxt = tabtxt.substring(0, tabtxt.indexOf("("));
		$activetab.text(tabtxt + "(" + newNuTicketsShown + ")");
	}
	InitViewingTechs(); //re-init the "eyes"
});

//signalr!
window.addEventListener("load", function () {
	//no signalr (user is non tech) - return
	if (typeof signalR === 'undefined') return;

	var connection = window._signalrConnection; //defined in utils.js

	//includes new tickets too
	connection.on("updateTicket", function (t) {
		//backward compat vars
		var statusId = t.StatusID, categoryId = t.CategoryID;

		//ticket is found in the grid? let update bagdes
		var $ticketRow = $('#ticketRow' + t.TicketID);
		if ($ticketRow.length) {
			$ticketRow.find(".new-badge").toggleFade(statusId == 1); // toggleFade defined in utils.js
			$ticketRow.find(".closed-badge").toggleFade(statusId == 3);
			$ticketRow.find(".cust-badge").toggleFade(t.UpdatedByUser && statusId != 3);
			$ticketRow.find(".tech-badge").toggleFade(t.UpdatedByPerformer && statusId != 3);
			$ticketRow.find(".fortech-badge").toggleFade(t.UpdatedForTech && statusId != 3);
			$ticketRow.find(".lastUpdatedTd span[data-ago-date]").attr("data-ago-date", new Date().toISOString().split('.').shift()); //set "data-ago-date", utils.js will do the rest. Split+Shift removes miliseconds and "Z"
			$ticketRow.find(".ticketLink").text(t.Subject); //update the subject in case it was a merge
			if (getStatusNameById(statusId)) //update status (if found)
				$ticketRow.find(".statusName").text(getStatusNameById(statusId));

			if (statusId == 3 && IndexPageModel.ListMode != "ShowAll") { removeFromGrid($ticketRow); return; } //remove closed tickets (unless its "show all")
		}

		//if filter is active - do nothing (todo?), if not - see if the ticet blongs to current view, then update it
		if ($('#btnResetFilter').length) return; //filter active? do nothing. we'll add stuff to future versions
		if ($('.tag.selected').length) return; //tag filter active? do nothing. may be we'll add in future versions

		var ticketBelongsToCurrentView = false; //check if ticket SHOULD be visible
		if (IndexPageModel.ListMode == "Unanswered") { ticketBelongsToCurrentView = (statusId != 3) && (statusId == 1 || t.UpdatedByUser || t.UpdatedForTech); }
		if (IndexPageModel.ListMode == "Unclosed") { ticketBelongsToCurrentView = (statusId != 3); }
		if (IndexPageModel.ListMode == "Unassigned") { if (!t.AssignedToUserID) ticketBelongsToCurrentView = true; }
		if (IndexPageModel.ListMode == "AssignedToUser") { ticketBelongsToCurrentView = (t.AssignedToUserID == IndexPageModel.CurrentUserId && statusId != 3); }
		if (IndexPageModel.ListMode == "ShowAll") { ticketBelongsToCurrentView = true; }
		if (IndexPageModel.CategoryIds.length) { if (!IndexPageModel.CategoryIds.includes(categoryId)) ticketBelongsToCurrentView = false; } // are we looking at particular category?
		if (IndexPageModel.SectionId) { // are we looking at particular section?
			if (!IndexPageModel.Categories.filter(function (c) { return c.CategoryID == categoryId && c.SectionID == IndexPageModel.SectionId }).length)
				ticketBelongsToCurrentView = false;
		}
		if (statusId == -1) ticketBelongsToCurrentView = false; //ticket being deleted

		if (!ticketBelongsToCurrentView) { //filter should NOT be here? remove it
			removeFromGrid($ticketRow); return;
		}

		if ($ticketRow.length && IndexPageModel.OrderByColumn == -17) { $ticketRow.prependTo($ticketRow.parent()); } //move ticket upstairs if sorted by "updated"

		if (!$ticketRow.length) { //ticket SHOULD be here but its NOT found in the grid - lets refresh the grid
			if ($("#hdnHasMoreTickets").val() != "true" || IndexPageModel.OrderByColumn == -17 || IndexPageModel.OrderByColumn == -4) { // unless ticket is below the fold
				//there's no "more" button, or tickets are sorted by "updated", or sorted by "date" and its a "new" ticket... And also there's a grid (not the "no tickets found" msg)
				if (($("#hdnHasMoreTickets").val() != "true" || IndexPageModel.OrderByColumn == -17 || (IndexPageModel.OrderByColumn == -4 && statusId == 1)) && $("#divTicketsGrid").length)
					RenderTicket(t.TicketID); //lets just reder the new tiket at the top
				else
					RefreshCurrentGrid();
			}
		}
	});

	connection.on("updateViewingTechs", function (ticketId, users) {
		updateViewingTechs(ticketId, users);
	});

	connection.on("setActiveChat", function (ticketId, isActive) {
		var $ticketRow = $('#ticketRow' + ticketId);
		if (!$ticketRow.length) return; //ticket is not currently on screen

		if (isActive)
			$(".live-badge", $ticketRow).fadeIn('fast');
		else
			$(".live-badge", $ticketRow).fadeOut('fast');
	});

	// Start the connection
	_signalrAfterConnect.then(function () {
		connection.invoke("SubscribeGrid");
	});

	//reconnect when connection dropped
	connection.onreconnected(function (connectionId) {
		if (IndexPageModel.ListMode == "ShowAll") return; //don't refresh "showall" page
		setTimeout(function () {
			window.location.reload(true);
		}, 25000 + Math.random() * 40000); //after 20 seconds so clients don't rush all at once after deploy, also randomize
	});
});

function getStatusNameById(statusId) {
	return $(`#statusId option[value=${statusId}]`).text(); //get from filter selector
}

//add notification bar
function areNotificationsDisabled() {
	if (typeof Notification === 'undefined') return false;
	return Notification.permission === "denied";
}

function removeFromGrid(el) {
	el.fadeRemove().then(function () {
		updateTicketCount(-1);
		// if no more tickets are left in the current view
		// we need to show an empty state
		if ($("#tblTickets tbody tr").length > 0) return; //still some tickets left

		//it was the last one. Hide the grid.
		$("#tblTickets").hide();
		$("#noTicketsMessage").fadeIn('fast');
	});
}

function updateTicketCount(increment) {
	var span = document.querySelector(".active2 span");
	if (span === null) return;

	var currentCount = Number(span.innerText);
	span.innerText = (currentCount + increment) + '';
}

function toggleCategories(el) {
	var catUl = $(el).nextAll('ul');
	var isCollapsed = catUl.is(':visible'); // visible before the toggle means it is going to be collapsed
	var sectionId = $(el).data('section-id');
	catUl.slideToggle('fast');

	var collapsedSections = $.cookie("collapsedSections");
	// backwards compatibility
	try {
		collapsedSections = JSON.parse(collapsedSections);
	} catch (e) {
		collapsedSections = [];
	}

	if (!isCollapsed) {
		var index = collapsedSections.indexOf(sectionId);
		if (index !== -1) {
			collapsedSections.splice(index, 1);
		}
	} else if (collapsedSections.indexOf(sectionId) === -1) {
		collapsedSections.push(sectionId);
	}

	$.cookie("collapsedSections", JSON.stringify(collapsedSections), { expires: 30 });
}