﻿//tiny color tool from jitbit

'use strict';

var ColorTool = new function () { //singleton pattern

	//takes color in any CSS-compatible format, return array of [R,G,B]
	//from https://stackoverflow.com/a/21966100/56621
	this.parseColor = function (input) {
		if (input.substr(0, 1) == "#") { //#HEX format
			var collen = (input.length - 1) / 3;
			var fact = [17, 1, 0.062272][collen - 1];
			return [
				Math.round(parseInt(input.substr(1, collen), 16) * fact),
				Math.round(parseInt(input.substr(1 + collen, collen), 16) * fact),
				Math.round(parseInt(input.substr(1 + 2 * collen, collen), 16) * fact)
			];
		}
		else if (input.indexOf("(") > -1) //rgb(1,2,3) format
			return input.split("(")[1].split(")")[0].split(",").map(Math.round);
		else //word format (edge can just return "transparent" or "grey")
			return [0, 0, 0]; //just return black
	}

	//brighntness in 0..255 format. 128 is the middle
	this.getBrightnessYIQ = function (rgb) {
		var r = rgb[0], g = rgb[1], b = rgb[2];
		return ((r * 299) + (g * 587) + (b * 114)) / 1000;
	}

	this.removeDisallowedColorStyles = function (selector, minBrightness, maxBrightness, isBackground) {
		var elements = document.querySelectorAll(selector);
		for (var i = 0; i < elements.length; i++) {
			var el = elements[i];
			if (isBackground)
				var brightness = this.getBrightnessYIQ(this.parseColor(getComputedStyle(el).backgroundColor));
			else
				var brightness = this.getBrightnessYIQ(this.parseColor(getComputedStyle(el).color));
			var isAllowed = (brightness >= minBrightness && brightness <= maxBrightness);
			if (!isAllowed) {
				if (isBackground) {
					el.style.removeProperty("background-color");
					el.style.removeProperty("background");
				}
				else {
					el.style.removeProperty("color");
					el.removeAttribute("color");
				}
			}
		}
	}
};